#!/bin/bash
# This script creates an Aegir platform, from the download up to the front-end.

#****************************************
# General variables
#****************************************

# The name of this file.
kplatforms_this_file=$(basename $0)

# The full path of this file.
kplatforms_this_path=$(dirname $(pwd)/$0)

# Today's date and time.
kplatforms_date="$(date +%Y).$(date +%m).$(date +%d)"
kplatforms_time="$(date +%H).$(date +%M).$(date +%S)"

# Requirements
kplatforms_req_user="aegir"
kplatforms_req_drush_version="4.5"
kplatforms_req_drush_make_version="2.3"

# Warning detection.
kplatforms_warning=0

#****************************************
# Usage
#****************************************

function kplatforms_echo_usage_and_exit {
  echo "Usage:"
  echo " $kplatforms_this_file [-a htpasswd_file] [-C | -c config_file] [-d drush_command] [-f] [-I | -i inclusions] [-M | -m other_makefiles] [-n platform_name] [-o hostmaster_instance_name] [-p platforms_path] [-s platform_suffix] [-v drupal_version] [-w web_server]"
  echo "See the 'README.txt' file for complete explanation."
  exit 1
}

#****************************************
# Arguments
#****************************************

# Processing arguments and defining variables.
while getopts ":a:Cc:d:fHhIi:Mm:n:o:p:s:v:w:" opt; do
  case $opt in

    a) # -a htpasswd_file
      kplatforms_htpasswd_file="$OPTARG"
      ;;

    C) # Do not read default configuration file.
      kplatforms_config_file="none"
      ;;

    c) # -c config_file
      kplatforms_config_file="$OPTARG"
      ;;

    d) # -d drush_command
      kplatforms_drush_command="$OPTARG"
      ;;

    f) # Using "drush --force-complete".
      kplatforms_force_complete="true"
      ;;

    H|h) # Help
      kplatforms_echo_usage_and_exit
      ;;

    I) # No inclusions.
      kplatforms_inclusions="none"
      ;;

    i) # -i inclusions
      kplatforms_inclusions="$OPTARG"
      ;;

    M) # Do not include default other makefile.
      kplatforms_other_makefiles="none"
      ;;

    m) # -m /path/to/makefile.make
      kplatforms_other_makefiles="$OPTARG"
      ;;

    n) # -n platform_name
      kplatforms_platform_name="$OPTARG"
      ;;

    o) # -m hostmaster_instance_name
      kplatforms_hostmaster_instance_name="$OPTARG"
      ;;

    p) # -p platforms_path
      kplatforms_platforms_path="$OPTARG"
      ;;

    s) # -s platform_suffix
      kplatforms_platform_suffix="$OPTARG"
      ;;

    v) # -v drupal_version
      kplatforms_drupal_version="$OPTARG"

      # Checking if Drupal version is supported.
      if [ $kplatforms_drupal_version != 7 -a $kplatforms_drupal_version != 6 -a $kplatforms_drupal_version != 5 ] ; then
        echo "ERROR: Version '$kplatforms_drupal_version' is not supported."
        echo "Version must be '7', '6' or '5'. Default is '7'."
        kplatforms_echo_usage_and_exit
      fi

      ;;

    w) # -w web_server
      kplatforms_web_server="$OPTARG"
      ;;

    \?) # Invalid option.
      echo "ERROR: Invalid option: -$OPTARG"
      kplatforms_echo_usage_and_exit
      ;;

    :) # Missing argument.
      echo "ERROR: Option -$OPTARG requires an argument."
      kplatforms_echo_usage_and_exit
      ;;

  esac
done

#****************************************
# Configuration file
#****************************************

function kplatforms_read_config_file {

  . $kplatforms_config_file

  if [ $? -eq 0 ]; then   # Test exit code.
    echo "OK: Configuration file '$kplatforms_config_file' read successfully."
  else
    echo "ERROR reading configuration file '$kplatforms_config_file'."
    kplatforms_echo_usage_and_exit
  fi

}

if [ -z $kplatforms_config_file ]; then   # Reading the default configuration file if no one has been defined.

  kplatforms_config_file="/var/aegir/config/kplatforms.conf"

  if [ -r "$kplatforms_config_file" ]; then
    kplatforms_read_config_file
  fi

elif [ "$kplatforms_config_file" != "none" ]; then   # Processing the configuration file from argument.

  if [ -r "$kplatforms_config_file" ]; then   # Checking if the file is readable.
    kplatforms_read_config_file
  else
    echo "ERROR: The configuration file '$kplatforms_config_file' cannot be read."
    kplatforms_echo_usage_and_exit
  fi

fi

#****************************************
# Default variables
#****************************************

# Defining variables if not already defined neither by argument, config file nor shell variable.

# Drupal core major version number.
if [ -z $kplatforms_drupal_version ]; then
  kplatforms_drupal_version=7
fi
echo "Variable kplatforms_drupal_version = '$kplatforms_drupal_version'"

# Platform name.
if [ -z $kplatforms_platform_name ]; then
  kplatforms_platform_name="drupal-$kplatforms_drupal_version"
fi
echo "Variable kplatforms_platform_name = '$kplatforms_platform_name'"

# Drush command.
if [ -z $kplatforms_drush_command ]; then
  kplatforms_drush_command="drush"
fi
echo "Variable kplatforms_drush_command = '$kplatforms_drush_command'"

# Hosmaster instance name.
# To use like "drush @hostmaster ...".
if [ -z $kplatforms_hostmaster_instance_name ]; then
  kplatforms_hostmaster_instance_name="hostmaster"
fi
echo "Variable kplatforms_hostmaster_instance_name = '$kplatforms_hostmaster_instance_name'"

# Inclusions
if [ -z $kplatforms_inclusions ]; then
  kplatforms_inclusions="modules themes libraries"
fi
echo "Variable kplatforms_inclusions = '$kplatforms_inclusions'"

# Platforms path, directory where to create platforms.
if [ -z $kplatforms_platforms_path ]; then
  kplatforms_platforms_path="/var/aegir/platforms"
fi
echo "Variable kplatforms_platforms_path = '$kplatforms_platforms_path'"

# Temporary directory where to build the stub makefile.
if [ -z $kplatforms_tmpdir ]; then
  kplatforms_tmpdir="/tmp"
fi
echo "Variable kplatforms_tmpdir = '$kplatforms_tmpdir'"

# Web server.
if [ -z $kplatforms_web_server ]; then
  kplatforms_web_server="master"
fi
echo "Variable kplatforms_web_server = '$kplatforms_web_server'"

#****************************************
# Checking requirements
#****************************************

# User

kplatforms_user=$(id -nu)
echo "Current user = '$kplatforms_user'"

if [ "$kplatforms_user" != "$kplatforms_req_user" ] ; then
  echo "ERROR: You must run '$kplatforms_this_file' as user '$kplatforms_req_user'."
  exit 1
fi

# Drush version

kplatforms_drush_version=$($kplatforms_drush_command --version | awk '{print $NF}')
echo "Current 'drush' version = '$kplatforms_drush_version'"

if [ $(echo "$kplatforms_drush_version < $kplatforms_req_drush_version" | bc) -eq 1 ]; then
  echo "ERROR: 'Drush' version must be '$kplatforms_req_drush_version' or higher."
  exit 1
fi

# Drush make version

kplatforms_drush_make_version=$($kplatforms_drush_command make --version | awk '{print $NF}')
echo "Current 'drush make' version = '$kplatforms_drush_make_version'"

if [ $(echo "$kplatforms_drush_make_version < $kplatforms_req_drush_make_version" | bc) -eq 1 ]; then
  echo "ERROR: 'Drush make' version must be '$kplatforms_req_drush_make_version' or higher."
  exit 1
fi

#****************************************
# Building platform information
#****************************************

# The name of the platform directory without any path.
if [ "$kplatforms_platform_suffix" = "" ]; then
  kplatforms_platform_directory="$kplatforms_platform_name-build-$kplatforms_date"
else
  kplatforms_platform_directory="$kplatforms_platform_name-build-$kplatforms_date-$kplatforms_platform_suffix"
fi
echo "Platform name = '$kplatforms_platform_directory'"

# Absolute path to the platform.
kplatforms_platform_path="$kplatforms_platforms_path/$kplatforms_platform_directory"
echo "Platform path = '$kplatforms_platform_path'"

#****************************************
# Building stub makefile with inclusions
#****************************************

# Defining the directory where to find the makefiles to include.
kplatforms_includir="$kplatforms_this_path/includes/$kplatforms_drupal_version"
echo "Includes directory = '$kplatforms_includir'"

# Defining the stub makefile with its full path.
kplatforms_stub_makefile="$kplatforms_tmpdir/kplatforms-stub_$kplatforms_platform_directory-$kplatforms_time.make"
echo "Stub makefile = '$kplatforms_stub_makefile'"

# Replacing upper case by lower case.
kplatforms_inclusions=$(echo $kplatforms_inclusions | tr '[:upper:]' '[:lower:]')

# Replacing comas by space.
kplatforms_inclusions=$(echo $kplatforms_inclusions | sed 's/,/ /g')

echo "Inclusions = '$kplatforms_inclusions'"

# Defining the core makefile.
kplatforms_core_makefile="$kplatforms_includir/core.make"

# Creating the stub makefile by including the core makefile.
echo "Including makefile '$kplatforms_core_makefile'"
cat $kplatforms_core_makefile >$kplatforms_stub_makefile

# Check if no inclusions is defined.
if [ "$kplatforms_inclusions" != "none" ]; then

  # Processing inclusions one by one.
  for kplatforms_inclusion in $kplatforms_inclusions; do

    # The makefile with its full path.
    kplatforms_inclusion_makefile="$kplatforms_includir/$kplatforms_inclusion.make"

    # Checking if we can read the makefile.
    if [ ! -r "$kplatforms_inclusion_makefile" ]; then
      echo "ERROR: The makefile '$kplatforms_inclusion_makefile' cannot be read."
      rm $kplatforms_stub_makefile
      kplatforms_echo_usage_and_exit
    fi

    # Including the makefile.
    echo "Including makefile '$kplatforms_inclusion_makefile'"
    cat $kplatforms_inclusion_makefile >>$kplatforms_stub_makefile

  done

fi

#****************************************
# Reducing items in the stub makefile
#****************************************

# This reduces the list of libraries in the platform if a list is given.

if [ ! -z $kplatforms_unwanted_libraries ]; then   # If a list of unwanted libraries is defined...

  # Replacing upper case by lower case.
  kplatforms_unwanted_libraries=$(echo $kplatforms_unwanted_libraries | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_unwanted_libraries=$(echo $kplatforms_unwanted_libraries | sed 's/,/ /g')

  echo "Unwanted libraries = '$kplatforms_unwanted_libraries'"

  # Disabling specified libraries in the stub makefile.
  for kplatforms_unwanted_library in $kplatforms_unwanted_libraries; do
    echo "Disabling $kplatforms_unwanted_library."
    sed -i "/^libraries\[$kplatforms_unwanted_library\]/ s/^/;/" $kplatforms_stub_makefile
  done

elif [ ! -z $kplatforms_wanted_libraries ]; then   # If a list of wanted libraries is defined...

  # Replacing upper case by lower case.
  kplatforms_wanted_libraries=$(echo $kplatforms_wanted_libraries | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_wanted_libraries=$(echo $kplatforms_wanted_libraries | sed 's/,/ /g')

  echo "Wanted libraries = '$kplatforms_wanted_libraries'"

  echo "Disabling all libraries in the stub makefile."
  sed -i "/^libraries/ s/^/;/" $kplatforms_stub_makefile

  # Enabling specified libraries in the stub makefile.
  for kplatforms_wanted_library in $kplatforms_wanted_libraries; do
    echo "Enabling $kplatforms_wanted_library."
    sed -i "/^;libraries\[$kplatforms_wanted_library\]/ s/^;//" $kplatforms_stub_makefile
  done

fi

# This reduces the list of projects in the platform if a list is given.

if [ ! -z $kplatforms_unwanted_projects ]; then   # If a list of unwanted projects is defined...

  # Replacing upper case by lower case.
  kplatforms_unwanted_projects=$(echo $kplatforms_unwanted_projects | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_unwanted_projects=$(echo $kplatforms_unwanted_projects | sed 's/,/ /g')

  echo "Unwanted projects = '$kplatforms_unwanted_projects'"

  # Disabling specified projects in the stub makefile.
  for kplatforms_unwanted_project in $kplatforms_unwanted_projects; do
    echo "Disabling $kplatforms_unwanted_project."
    sed -i "/^projects\[$kplatforms_unwanted_project\]/ s/^/;/" $kplatforms_stub_makefile
  done

elif [ ! -z $kplatforms_wanted_projects ]; then   # If a list of wanted projects is defined...

  # Replacing upper case by lower case.
  kplatforms_wanted_projects=$(echo $kplatforms_wanted_projects | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_wanted_projects=$(echo $kplatforms_wanted_projects | sed 's/,/ /g')

  echo "Wanted projects = '$kplatforms_wanted_projects'"

  echo "Disabling all projects in the stub makefile."
  sed -i "/^projects/ s/^/;/" $kplatforms_stub_makefile

  echo "Enabling Drupal core."
  sed -i "/^;projects\[drupal\]\|^;projects\[pressflow\]/ s/^;//" $kplatforms_stub_makefile

  # Enabling specified projects in the stub makefile.
  for kplatforms_wanted_project in $kplatforms_wanted_projects; do
    echo "Enabling $kplatforms_wanted_project."
    sed -i "/^;projects\[$kplatforms_wanted_project\]/ s/^;//" $kplatforms_stub_makefile
  done

fi

#****************************************
# Adding other makefiles
#****************************************

function kplatforms_include_other_makefiles {

  # Replacing comas by space.
  kplatforms_other_makefiles=$(echo $kplatforms_other_makefiles | sed 's/,/ /g')

  # Processing makefiles one by one.
  for kplatforms_other_makefile in $kplatforms_other_makefiles; do

    if [ -r "$kplatforms_other_makefile" ]; then

      echo "Including makefile '$kplatforms_other_makefile'."
      cat $kplatforms_other_makefile >>$kplatforms_stub_makefile

      if [ $? -ne 0 ]; then   # Test exit code.
        echo "ERROR including makefile '$kplatforms_other_makefile'."
        kplatforms_echo_usage_and_exit
      fi

    else
      echo "ERROR: The makefile '$kplatforms_other_makefile' cannot be read."
      rm $kplatforms_stub_makefile
      kplatforms_echo_usage_and_exit
    fi

  done

}

if [ -z $kplatforms_other_makefiles ]; then   # Including default makefile if no one has been defined.
  kplatforms_other_makefiles="/var/aegir/config/kplatforms.make"

  if [ -r "$kplatforms_other_makefiles" ]; then
    kplatforms_include_other_makefiles
  fi

elif [ "$kplatforms_other_makefiles" != "none" ]; then   # Including makefiles from argument.
  kplatforms_include_other_makefiles
fi

#****************************************
# Drush commands
#****************************************

# Using "drush --force-complete".
kplatforms_force_complete=$(echo $kplatforms_force_complete | tr '[:upper:]' '[:lower:]')   # Replacing upper case by lower case.
if [ "$kplatforms_force_complete" = "true" ]; then
  kplatforms_drush_command="$kplatforms_drush_command --force-complete"
fi
echo "Drush command = '$kplatforms_drush_command'"

# Creating platform.
$kplatforms_drush_command make $kplatforms_stub_makefile $kplatforms_platform_path && \
  $kplatforms_drush_command provision-save @platform_$kplatforms_platform_directory \
    --context_type=platform \
    --web_server=@server_$kplatforms_web_server \
    --root=$kplatforms_platform_path \
    --makefile=$kplatforms_stub_makefile && \
  $kplatforms_drush_command @$kplatforms_hostmaster_instance_name hosting-import @platform_$kplatforms_platform_directory

# Test exit code.
if [ $? -eq 0 ]; then
  echo "OK: Platform created successfully in '$kplatforms_platform_path'."
else
  echo "ERROR: Platform not created successfully."
  exit 1
fi

#****************************************
# Access restricted by users (htauth)
#****************************************

if [ "$kplatforms_htpasswd_file" != "" ]; then

  # Checking if we can read the htpasswd file.
  if [ ! -r "$kplatforms_htpasswd_file" ]; then
    echo "WARNING: The htpasswd file cannot be read."
    echo "Use: 'htpasswd -b -c $kplatforms_htpasswd_file username password'."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

  # Defining variables.
  kplatforms_htaccess_file="$kplatforms_platform_path/.htaccess"
  kplatforms_htaccess_file_head="$kplatforms_platform_path/.htaccess.head"
  kplatforms_htaccess_file_tmp="$kplatforms_platform_path/.htaccess.tmp"

  # Editing htaccess file.
  echo "# Access restricted by users" >$kplatforms_htaccess_file_head && \
    echo "AuthUserFile $kplatforms_htpasswd_file" >>$kplatforms_htaccess_file_head && \
    echo "AuthName 'Authorized people only'" >>$kplatforms_htaccess_file_head && \
    echo "AuthType Basic" >>$kplatforms_htaccess_file_head && \
    echo "require valid-user" >>$kplatforms_htaccess_file_head && \
    echo "" >>$kplatforms_htaccess_file_head && \
    cat $kplatforms_htaccess_file_head $kplatforms_htaccess_file >$kplatforms_htaccess_file_tmp && \
    mv $kplatforms_htaccess_file_tmp $kplatforms_htaccess_file && \
    rm $kplatforms_htaccess_file_head

  # Test exit code.
  if [ $? -eq 0 ]; then
    echo "OK: Sites on this platform restricted successfully with '$kplatforms_htpasswd_file'."
  else
    echo "ERROR: Sites on this platform not restricted successfully."
    exit 1
  fi

fi

#****************************************
# Platform improvements
#****************************************

# Favicon, for flavour.
cp $kplatforms_this_path/favicon.ico $kplatforms_platform_path/
if [ $? -eq 0 ]; then
  echo "OK: Favicon file copied successfully."
else
  echo "WARNING: Favicon file not copied successfully."
  kplatforms_warning=`expr $kplatforms_warning + 1`
fi

# Making changes as per the Drupal core version.
if [ $kplatforms_drupal_version -eq 6 ]; then

  # Making changes as per the inclusions.
  for kplatforms_inclusion in $kplatforms_inclusions; do
    case $kplatforms_inclusion in

      libraries)

        # TCPDF library for the Drupal contrib module "Printer, e-mail and PDF versions" (print).
        # Directories non-writable permissions.

          # Web server group:
          # Use www-data for Debian and Ubuntu.
          # Use apache for RedHat and CentOS.
          kplatforms_webserver_grp="www-data"

          # TODO add test for RedHat
          if [ -x /usr/bin/lsb_release ]; then
            kplatforms_os=$(lsb_release -i | cut -f 2)
            if [ "$kplatforms_os" = "CentOS" ]; then
              kplatforms_webserver_grp=apache
            fi
          fi

          # Modifying permissions and group.
          chmod g+w $kplatforms_platform_path/sites/all/libraries/tcpdf/cache $kplatforms_platform_path/sites/all/libraries/tcpdf/images && \
            chgrp $kplatforms_webserver_grp $kplatforms_platform_path/sites/all/libraries/tcpdf/cache $kplatforms_platform_path/sites/all/libraries/tcpdf/images

          # Test exit code.
          if [ $? -eq 0 ]; then
            echo "OK: TCPDF chmod and chgrp."
          else
            echo "WARNING: TCPDF chmod and chgrp not done successfully."
            kplatforms_warning=`expr $kplatforms_warning + 1`
          fi

      ;;   # End of libraries.

    esac
  done     # End of inclusions.

fi         # End of Drupal 6.

#****************************************
# Exiting
#****************************************

if [ $kplatforms_warning -eq 0 ]; then
  echo "OK: '$kplatforms_this_file' terminated successfully."
  exit 0
elif [ $kplatforms_warning -eq 1 ]; then
  echo "WARNING: '$kplatforms_this_file' terminated with $kplatforms_warning warning."
  exit $kplatforms_warning
else
  echo "WARNING: '$kplatforms_this_file' terminated with $kplatforms_warning warnings."
  exit $kplatforms_warning
fi

#****************************************
# End
#****************************************
